/*
One of those Weeks Day 7
Copyright (c) 2007-2009, 2011-2012 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.converse
{
	// Adobe Imports
	import flash.events.Event;
	import flash.events.EventDispatcher;
//	import flash.events.MouseEvent;

	/**
	 * This early version of this class is essentially a glorified structure, 
	 * though future versions may have more functionality.
	 * @author Billy D. Spelchan
	 * @version 0.5
	 * Copyright (c) 2007 Blazing Games Inc. All Rights Reserved.
	*/
	public class ConversationHandler
	extends EventDispatcher
	{
		// ***** CONSTANTS ***************************************************
		
		public static const EXIT_TOPIC = new ConversationTopic(
				"Exit", "Exit the conversation", 
				"Exit should not be used for displaying text", 0, 0);
				
		// ***** VARIABLES ***************************************************

		protected var _speach:ISpeach;
		protected var _script:IScriptCommand;
		protected var _topics:Object;
		protected var _shownTopics:Array;
		protected var _lastSelectedTopic:ConversationTopic = EXIT_TOPIC;

		// ***** CONSTRUCTOR *************************************************

		public function ConversationHandler(speach:ISpeach, script:IScriptCommand)
		{
			_speach = speach;
			_speach.addEventListener(Converse.SPEACH_EVENT, clickHandler)
			_script = script;
			_topics = new Object();
			_shownTopics = new Array();
		}

		// ***** Event Handlers *********************************************
		
		protected function clickHandler(evt:Event)
		{
			var n:int = _speach.getLastResponse();
			_speach.clearDisplay();
			_lastSelectedTopic = _shownTopics[n];
			var evnt:Event = new Event(Converse.TOPIC_EVENT);
			this.dispatchEvent(evnt);
		}
		
		public function getLastTopic():ConversationTopic
		{
			return _lastSelectedTopic;
		}
		
		// ***** Methods - Topic Handling ***********************************

		public function addTopic(top:ConversationTopic)
		{
			_topics[top.topic] = top;
		}

		public function getTopic(s:String):ConversationTopic
		{
			var top:ConversationTopic = _topics[s];
			if (top == null) {
				top = new ConversationTopic(s, s + " not implemented!",
				"ERROR. The topic " + s + " has not yet been implemented!");
				addTopic(top);
			}
			return top;
		}
		
		public function showTopic(s:String)
		{
			var cntr:int;
			var top:ConversationTopic = getTopic(s);

			// run the conversation code if any
			if (top.displayCommands != null) {
				for (cntr = 0; cntr < top.displayCommands.length; ++cntr)
					_script.parseCommand(top.displayCommands[cntr]);
			}

			// now build the display
			_speach.clearDisplay();
			_speach.setDialog(top.text, false, top.topicType);
			if (top.subtopics == null) {
				_speach.addResponse(EXIT_TOPIC.subject,0, EXIT_TOPIC.topicType);
				_shownTopics[0] = EXIT_TOPIC;
			} else {
				var sub:ConversationTopic;
				for (cntr = 0; cntr < top.subtopics.length; ++cntr) {
					sub = getTopic(top.subtopics[cntr]);
					_speach.addResponse(sub.subject,
							cntr, sub.subjectType);
					_shownTopics[cntr] = sub;
				}
			}
			
			_speach.startDisplay();
		}

	} // end class
} // end package

