/*
One of those Weeks Episode 44: Alone in the Lab
Copyright (c) 2008,2011-2012 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.ep44
{
	// Adobe Imports
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.Event;
	import flash.events.MouseEvent;

	// Blazing Games Imports
	import com.blazinggames.ootw.converse.*;
	import com.blazinggames.ootw.inventory.*;

	public class SecretLab
	extends MovieClip
	{
// ***** CONSTANTS **********************************************************

		public static const DOOR_OPEN_EVENT = "DoorOpenEvent";

		private static const LOCK_KEYS:Array = new Array(
			new Array(11,6, 3,3, 0,1),
			new Array(3,3, 7,4, 0,3),
			new Array(7,1, 11,1, 0,8)
		);

// ***** VARIABLES **********************************************************

		private var _chemButtonList:Array;
		private var _chemStrips:Array;
		private var _inventory:IInventoryBox;
		private var _haveStrips:Boolean;
		private var	_stripInserted:int;
		private var _locksLocked:Array;
		private var _locks:Array;
		
// ***** CONSTRUCTOR ********************************************************

		public function SecretLab()
		{
			var cntr:int;
			
			// Buttons for the chemicals
			_chemButtonList = new Array(
				_chem01_movie, _chem02_movie, _chem03_movie, _chem04_movie,
				_chem05_movie, _chem06_movie, _chem07_movie, _chem08_movie,
				_chem09_movie, _chem10_movie, _chem11_movie, _chem12_movie
			);
			
			for (cntr = 0; cntr < _chemButtonList.length; ++cntr)
				_chemButtonList[cntr].addEventListener(MouseEvent.MOUSE_UP, chemHandler);
			// Mixing machine buttons
			_mix_btn.addEventListener(MouseEvent.MOUSE_UP, handleMix);
			_clear_btn.addEventListener(MouseEvent.MOUSE_UP, handleClear);
			// chemical strips
			_chemStrips = new Array(
					new BasicInventoryItem(null, new ChemicalStrip(), "1"),
					new BasicInventoryItem(null, new ChemicalStrip(), "2"),
					new BasicInventoryItem(null, new ChemicalStrip(), "3"));			
			_strips_btn.addEventListener(MouseEvent.MOUSE_UP, getStrips);
			_insert_btn.addEventListener(MouseEvent.MOUSE_UP, handleInsert);

			// door locks
			_lock1_btn.addEventListener(MouseEvent.MOUSE_UP, handleLock);
			_lock2_btn.addEventListener(MouseEvent.MOUSE_UP, handleLock);
			_lock3_btn.addEventListener(MouseEvent.MOUSE_UP, handleLock);
			_locks = new Array(_lock1_btn, _lock2_btn, _lock3_btn);
			_locksLocked = new Array(true, true, true);
			
		}

// ***** Functions - Event handlers ******************************************

		/**
		 * When chemical button pressed, add it to the mix
		 */
		private function chemHandler(event:Event):void 
		{
			var chemTarget:int = 0;
			
			for (var cntr:int = 0; cntr < _chemButtonList.length; ++cntr)
				if (event.target == _chemButtonList[cntr])
					chemTarget = cntr + 1;
			if (chemTarget > 0)
				_mix_movie.addChemical(chemTarget);
		}

		/**
		 * Puts chemical mix onto strip
		 */
		private function handleMix(event:Event)
		{
			var chemStrip:ChemicalStrip;
			
			_strip_movie.cloneMix(_mix_movie);
			if (_stripInserted > 0) {
				chemStrip = ChemicalStrip(_chemStrips[_stripInserted-1].getIcon());
				chemStrip.cloneMix(_mix_movie);
			}
			_mix_movie.clearMix();
		}
		
		/**
		 * Clears current chemical mix
		 */
		private function handleClear(event:Event)
		{
			_mix_movie.clearMix();
		}
		
		/**
		 * lets player pick up the chemical strips lying on the machine
		 */
		private function getStrips(event:Event)
		{
			if (_inventory == null)
				return;
			_haveStrips = true;
			_inventory.clearInventory();
			_inventory.addInvItem(_chemStrips[0]);
			_inventory.addInvItem(_chemStrips[1]);
			_inventory.addInvItem(_chemStrips[2]);
			updateLab();
		}
		
		/**
		 * Lets player insert strip into machine so they can create a mix
		 */
		private function handleInsert(event:Event)
		{
			if ( ! _haveStrips)
				return;
			if (_stripInserted > 0) {
				// treating as pickup
				_inventory.setInventoryState(InventoryConst.INVSTATE_NORMAL);
				_inventory.addInvItem(_chemStrips[_stripInserted-1]);
				_stripInserted = 0;
			} else {
				var strip = _inventory.getSelectedItem();
				if (strip != null) {
					_inventory.removeInvItem(strip);
					_stripInserted = int(strip.getItemName());
					_strip_movie.cloneMix(strip.getIcon()._mix_movie);
				}
			}
			updateLab();
		}

		/**
		 * unlocks lock/door if sequences match. 
		 */
		public function handleLock(evt:Event)
		{
			var lockID:int = -1;
			
			for (var cntr:int = 0; cntr < _locks.length; ++cntr)
				if (evt.target == _locks[cntr])
					lockID = cntr;
			if (lockID < 0)
				return;
			var strip = _inventory.getSelectedItem();
			if (strip == null)
				return;
			_locksLocked[lockID] = ! strip.getIcon().validateMix(LOCK_KEYS[lockID]);
			strip.getIcon()._mix_movie.clearMix();
			_inventory.setInventoryState(InventoryConst.INVSTATE_NORMAL);
			updateLab();
		}

// ***** Functions - General control ****************************************

		/**
		 * Restarts the game. This consists of clearing the inventory, 
		 * resetting the strips, and redrawing the lab.
		 */
		public function restartGame(inv:IInventoryBox)
		{
			_inventory = inv;
			inv.clearInventory();
			_haveStrips = false;
			_stripInserted = 0;
			updateLab();
		}
		
		/**
		 * Updates the lab sprites to reflect the current state of the game.
		 */
		public function updateLab()
		{
			_strips_btn.visible = !_haveStrips;
			_strip_movie.visible = (_stripInserted > 0);
			_mix_btn.visible = (_stripInserted > 0);
			var gameInProgress:Boolean = false;
			for (var cntr:int = 0; cntr < _locksLocked.length; ++cntr) {
				_locks[cntr].visible = _locksLocked[cntr];
				gameInProgress = (gameInProgress || _locksLocked[cntr]);
			}
			_door_movie.visible = gameInProgress;
			if (!gameInProgress) {
				var evt:Event = new Event(DOOR_OPEN_EVENT);
				this.dispatchEvent(evt);
			}
		}
	} // end class
} // end package

