/*
One of those Weeks Day 7
Copyright (c) 2007-2009, 2011-2012 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.inventory
{
	// Adobe Imports
	import flash.display.Sprite;
	import flash.filters.GlowFilter;

	/**
	* A simple implementation of the IInventoryItem interface that should be
	* good enough for general inventory system use. This class is not 
	* derrived from any other class and is really designed to be used as
	* a variable in the implementation of an inventory item or as just as a
	* stand alone class describing inventory items used in the game.
	*
	* @author Billy D. Spelchan
	* @version 1.0
	*/
	public class BasicInventoryItem
	implements IInventoryItem
	{
// ***** CONSTANTS **********************************************************

// ***** VARIAIBLES *********************************************************

		public var _cursorSprite:Sprite;
		public var _iconSprite:Sprite;
		public var _itemName:String;
		public var _info:Object;
		protected var _highlights:Array;

// ***** CONSTRUCTOR ********************************************************

		public function BasicInventoryItem (
				cursor:Sprite, icon:Sprite, itemName:String,
				info:Object = null )
		{
			_cursorSprite = cursor;
			_iconSprite = icon;
			_itemName = itemName;
			if (info == null)
				_info = new Object();
			else
				_info = info;
			_highlights = new Array(3);
			
			var baseFilter:Array = _iconSprite.filters;
			if (baseFilter == null) 
				baseFilter = new Array();
			for (var cntr:int = 0; cntr < 3; ++cntr) {
				_highlights[cntr] = new Array();
				_highlights[cntr].concat(baseFilter);
			}
			_highlights[1].push(new GlowFilter(0xFF0000, 1, 16,16));
			_highlights[2].push(new GlowFilter(0x00FF00, 1, 16,16));
		}

// ***** Functions - Setters ************************************************

		/**
		* @param spr The sprite to use as a cursor when the sprite is used.
		*/
		public function setCursor(spr:Sprite)
		{
			_cursorSprite = spr;
		}

		/**
		* @param spr the sprite used to represent the object in the inventory.
		*/
		public function setIcon(spr:Sprite)
		{
			_iconSprite = spr;
		}
		
		/**
		* @param itemName the name of the item
		*/
		public function setItemName(itemName:String)
		{
			_itemName = itemName;
		}
		
		/**
		* @param info The information object for this inventory item
		*/
		public function setInfo(info:Object)
		{
			_info = info;
		}

// ***** Functions - IInventoryItem required ********************************

		/**
		* @return The sprite to use as a cursor when the sprite is being used.
		*/
		public function getCursor():Sprite
		{
			return _cursorSprite;
		}

		/**
		* @return the sprite used to represent the object in the inventory box.
		*/
		public function getIcon():Sprite
		{
			return _iconSprite;
		}
		
		/**
		* @return the name of the item
		*/
		public function getItemName():String
		{
			return _itemName;
		}

		/**
		* @return The object that holds additional item information
		*/
		public function getItemInfo():Object
		{
			return _info;
		}

		/**
		* @param htype sets the highlight state
		*/
		public function setHighlight(htype:uint)
		{
			var indx:uint = Math.min(htype, 2);
			_iconSprite.filters = _highlights[indx];
		}

	} // end interface
} // end package
